# -*- coding: utf-8 -*-
# Author: Manuel de la Pena <manuel@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Controllers with the logic of the UI."""

import os
import StringIO
import tempfile

from PIL import Image
from PyQt4.QtGui import QMessageBox, QWizard, QPixmap
from PyQt4.QtCore import QUrl
from twisted.internet.defer import inlineCallbacks, returnValue

from ubuntu_sso import NO_OP
from ubuntu_sso.logger import setup_logging
from ubuntu_sso.main.windows import UbuntuSSOClient
from ubuntu_sso.utils.ui import (
    CAPTCHA_SOLUTION_ENTRY,
    EMAIL1_ENTRY,
    EMAIL2_ENTRY,
    EMAIL_LABEL,
    EMAIL_MISMATCH,
    EMAIL_INVALID,
    ERROR,
    EXISTING_ACCOUNT_CHOICE_BUTTON,
    FORGOTTEN_PASSWORD_BUTTON,
    JOIN_HEADER_LABEL,
    NAME_ENTRY,
    PASSWORD1_ENTRY,
    PASSWORD2_ENTRY,
    PASSWORD_HELP,
    PASSWORD_MISMATCH,
    PASSWORD_TOO_WEAK,
    REQUEST_PASSWORD_TOKEN_LABEL,
    RESET_PASSWORD,
    RESET_CODE_ENTRY,
    REQUEST_PASSWORD_TOKEN_WRONG_EMAIL,
    REQUEST_PASSWORD_TOKEN_TECH_ERROR,
    SET_UP_ACCOUNT_CHOICE_BUTTON,
    SET_UP_ACCOUNT_BUTTON,
    SIGN_IN_BUTTON,
    SURNAME_ENTRY,
    SUCCESS,
    TC_BUTTON,
    TRY_AGAIN_BUTTON,
    VERIFY_EMAIL_TITLE,
    VERIFY_EMAIL_CONTENT,
    YES_TO_TC,
    get_password_strength,
    is_min_required_password,
    is_correct_email)


logger = setup_logging('ubuntu_sso.controllers')

FAKE_URL = '<a href="http://one.ubuntu.com">%s</a>'
# pylint: disable=W0511
# disabled warnings about TODO comments


class BackendController(object):
    """Represent a controller that talks with the sso backend."""

    def __init__(self):
        """Create a new instance."""
        self.root = None
        self.backends = []

    def __del__(self):
        """Clean the resources."""
        logger.info('Unregistering %s backends', len(self.backends))
        for backend in self.backends:
            backend.unregister_to_signals()
        if self.root is not None:
            logger.info('Disconnecting from root.')
            self.root.disconnect()

    @inlineCallbacks
    def get_backend(self):
        """Return the backend used by the controller."""
        # get the back end from the root
        if self.root is None:
            self.root = UbuntuSSOClient()
            self.root = yield self.root.connect()
        backend = self.root.sso_login
        yield backend.register_to_signals()
        self.backends.append(backend)
        logger.debug('Number of current backends in the controller: %s',
                     len(self.backends))
        returnValue(backend)


class ChooseSignInController(object):
    """Controlled to the ChooseSignIn view/widget."""

    def __init__(self, title=''):
        """Create a new instance to manage the view."""
        self._title = title

    # use an ugly name just so have a simlar api as found in PyQt
    #pylint: disable=C0103
    def setupUi(self, view):
        """Perform the required actions to set up the ui."""
        view.setTitle(self._title)
        self._set_up_translated_strings(view)
        self._connect_buttons(view)
    #pylint: enable=C0103

    def _set_up_translated_strings(self, view):
        """Set the correct strings for the UI."""
        logger.debug('ChooseSignInController._set_up_translated_strings')
        view.existing_account_button.setText(EXISTING_ACCOUNT_CHOICE_BUTTON)
        view.new_account_button.setText(SET_UP_ACCOUNT_CHOICE_BUTTON)

    def _connect_buttons(self, view):
        """Connect the buttons to the actions to perform."""
        logger.debug('ChooseSignInController._connect_buttons')
        view.existing_account_button.clicked.connect(
                                        lambda: self._set_next_existing(view))
        view.new_account_button.clicked.connect(
                                             lambda: self._set_next_new(view))

    def _set_next_existing(self, view):
        """Set the next id and fire signal."""
        logger.debug('ChooseSignInController._set_next_existing')
        view.next = view.wizard().current_user_page_id
        view.wizard().next()

    def _set_next_new(self, view):
        """Set the next id and fire signal."""
        logger.debug('ChooseSignInController._set_next_new')
        view.next = view.wizard().setup_account_page_id
        view.wizard().next()


class CurrentUserController(BackendController):
    """Controller used in the view that is used to allow the signin."""

    def __init__(self, backend=None, title='', message_box=None):
        """Create a new instance."""
        super(CurrentUserController, self).__init__()
        if message_box is None:
            message_box = QMessageBox
        self.message_box = message_box
        self._title = title

    def _set_translated_strings(self, view):
        """Set the translated strings."""
        logger.debug('CurrentUserController._set_translated_strings')
        view.email_edit.setPlaceholderText(EMAIL1_ENTRY)
        view.password_edit.setPlaceholderText(PASSWORD1_ENTRY)
        view.forgot_password_label.setText(FAKE_URL % FORGOTTEN_PASSWORD_BUTTON)
        view.sign_in_button.setText(SIGN_IN_BUTTON)

    def _connect_ui(self, view, backend):
        """Connect the buttons to perform actions."""
        logger.debug('CurrentUserController._connect_buttons')
        view.sign_in_button.clicked.connect(lambda: self.login(view, backend))
        # lets add call backs to be execute for the calls we are interested
        backend.on_login_error_cb = lambda app, error:\
                                        self.on_login_error(view, app, error)
        backend.on_logged_in_cb = lambda app, result:\
                                        self.on_logged_in(view, app, result)
        view.forgot_password_label.linkActivated.connect(lambda link:\
                                        self.on_forgotten_password(view))

    def login(self, view, backend):
        """Perform the login using the backend."""
        logger.debug('CurrentUserController.login')
        # grab the data from the view and call the backend
        d = backend.login(view.wizard().app_name, view.email, view.password)
        d.addErrback(lambda e: self.on_login_error(view, e))

    def on_login_error(self, view, error):
        """There was an error when login in."""
        # let the user know
        logger.error('Got error when login %s, error: %s', app_name, error)
        self.message_box.critical(view, view.wizard().app_name, str(error))

    def on_logged_in(self, view, app_name, result):
        """We managed to log in."""
        logger.info('Logged in for %s', app_name)
        view.wizard().loginSuccess.emit(app_name, view.email)
        logger.debug('Wizard.loginSuccess emitted.')

    def on_forgotten_password(self, view):
        """Show the user the forgotten password page."""
        logger.info('Forgotten password')
        view.next = view.wizard().forgotten_password_page_id
        view.wizard().next()

    # use an ugly name just so have a simlar api as found in PyQt
    #pylint: disable=C0103
    @inlineCallbacks
    def setupUi(self, view):
        """Setup the view."""
        backend = yield self.get_backend()
        view.setTitle(self._title)
        self._set_translated_strings(view)
        self._connect_ui(view, backend)
    #pylint: enable=C0103


class SetUpAccountController(BackendController):
    """Conroller for the setup account view."""

    def __init__(self,  message_box=None):
        """Create a new instance."""
        super(SetUpAccountController, self).__init__()
        if message_box is None:
            message_box = QMessageBox
        self.message_box = message_box

    def _set_translated_strings(self, view):
        """Set the different gettext translated strings."""
        logger.debug('SetUpAccountController._set_translated_strings')
        # set the translated string
        view.name_edit.setPlaceholderText(NAME_ENTRY)
        view.last_name_edit.setPlaceholderText(SURNAME_ENTRY)
        view.email_edit.setPlaceholderText(EMAIL1_ENTRY)
        view.confirm_email_edit.setPlaceholderText(EMAIL2_ENTRY)
        view.password_edit.setPlaceholderText(PASSWORD1_ENTRY)
        view.confirm_password_edit.setPlaceholderText(PASSWORD2_ENTRY)
        view.password_info_label.setText(PASSWORD_HELP)
        view.captcha_solution_edit.setPlaceholderText(CAPTCHA_SOLUTION_ENTRY)
        view.terms_and_conditions_check_box.setText(
                            YES_TO_TC % {'app_name': view.wizard().app_name})
        view.terms_and_conditions_button.setText(TC_BUTTON)
        view.set_up_button.setText(SET_UP_ACCOUNT_BUTTON)

    def _set_line_edits_validations(self, view):
        """Set the validations to be performed on the edits."""
        logger.debug('SetUpAccountController._set_line_edits_validations')
        view.set_line_edit_validation_rule(view.email_edit, is_correct_email)
        # set the validation rule for the email confirmation
        view.set_line_edit_validation_rule(view.confirm_email_edit,
                         lambda x: self.is_correct_email_confirmation(x, view))
        # connect the changed text of the password to trigger a changed text
        # in the confirm so that the validation is redone
        view.email_edit.textChanged.connect(
                                      view.confirm_email_edit.textChanged.emit)
        view.set_line_edit_validation_rule(view.password_edit,
                                           is_min_required_password)
        view.set_line_edit_validation_rule(view.confirm_password_edit,
                      lambda x: self.is_correct_password_confirmation(x, view))
        # same as the above case, lets connect a signal to a signal
        view.password_edit.textChanged.connect(
                                   view.confirm_password_edit.textChanged.emit)

    def _connect_ui_elements(self, view, backend):
        """Set the connection of signals."""
        logger.debug('SetUpAccountController._connect_ui_elements')
        view.terms_and_conditions_button.clicked.connect(
                                               lambda: self.set_next_tos(view))
        view.set_up_button.clicked.connect(lambda: self.set_next_validation(
                                                                view, backend))
        view.password_edit.textChanged.connect(
                              lambda x: self.update_password_strength(x, view))
        view.terms_and_conditions_check_box.stateChanged.connect(
                                                 view.set_up_button.setEnabled)
        view.captcha_refresh_label.linkActivated.connect(lambda url:\
                                          self._refresh_captcha(view, backend))
        # set the callbacks for the captcha generation
        backend.on_captcha_generated_cb = lambda app, result:\
                                self.on_captcha_generated(view, app, result)
        backend.on_captcha_generation_error_cb = lambda app, error:\
                            self.on_captcha_generation_error(view, app, error)
        backend.on_user_registered_cb = lambda app, result:\
                            self.on_user_registered(view, app, result)
        backend.on_user_registration_error_cb = lambda app, error:\
                            self.on_user_registration_error(view, app, error)
        return backend

    def _refresh_captcha(self, view, backend):
        """Refresh the captcha image shown in the ui."""
        logger.debug('SetUpAccountController._refresh_captcha')
        # lets clean behind us, do we have the old file arround?
        old_file = None
        if view.captcha_file and os.path.exists(view.captcha_file):
            old_file = view.captcha_file
        fd = tempfile.TemporaryFile(mode='r')
        file_name = fd.name
        view.captcha_file = file_name
        d = backend.generate_captcha(view.wizard().app_name, file_name)
        if old_file:
            d.addCallback(lambda x: os.unlink(old_file))
        d.addErrback(lambda e: self.on_captcha_generation_error(view, e))

    def _set_titles(self, view):
        """Set the diff titles of the view."""
        logger.debug('SetUpAccountController._set_titles')
        view.setTitle(JOIN_HEADER_LABEL % {'app_name': view.wizard().app_name})
        view.setSubTitle(view.wizard().help_text)

    def on_captcha_generated(self, view, app_name, result):
        """A new image was generated."""
        logger.debug('SetUpAccountController.on_captcha_generated')
        view.captcha_id = result
        # HACK: First, let me apologize before hand, you can mention my mother
        # if needed I would do the same (mandel)
        # In an ideal world we could use the Qt plug-in for the images so that
        # we could load jpgs etc.. but this won't work when the app has been
        # brozen win py2exe using bundle_files=1
        # The main issue is that Qt will complain about the thread not being
        # the correct one when performing a moveToThread operation which is
        # done either by a setParent or something within the qtreactor, PIL
        # in this case does solve the issue. Sorry :(
        pil_image = Image.open(view.captcha_file)
        string_io = StringIO.StringIO()
        pil_image.save(string_io, format='png')
        pixmap_image = QPixmap()
        pixmap_image.loadFromData(string_io.getvalue())
        view.captcha_image = pixmap_image

    def on_captcha_generation_error(self, view, error):
        """An error ocurred."""
        logger.debug('SetUpAccountController.on_captcha_generation_error')
        self.message_box.critical(view, view.wizard().app_name, str(error))

    def on_user_registration_error(self, view, app_name, error):
        """Let the user know we could not register."""
        logger.debug('SetUpAccountController.on_user_registration_error')
        # error are returned as a dict with the data we want to show.
        self.message_box.critical(view, error['errtype'], error['email'])

    def on_user_registered(self, view, app_name, result):
        """Execute when the user did register."""
        logger.debug('SetUpAccountController.on_user_registered')
        view.wizard().registrationSuccess.emit(app_name, view.email)

    def set_next_tos(self, view):
        """Set the tos page as the next one."""
        logger.debug('SetUpAccountController.set_next_tos')
        view.next = view.wizard().tos_page_id
        view.wizard().next()

    def validate_form(self, view):
        """Validate the info of the form and return an error."""
        logger.debug('SetUpAccountController.validate_form')
        if not is_correct_email(view.email):
            self.message_box.critical(view, view.wizard().app_name,
                                      EMAIL_INVALID)
        if view.email_edit.text() != view.confirm_email_edit.text():
            self.message_box.critical(view, view.wizard().app_name,
                                      EMAIL_MISMATCH)
            return False
        if not is_min_required_password(str(view.password_edit.text())):
            self.message_box.critical(view, view.wizard().app_name,
                                      PASSWORD_TOO_WEAK)
            return False
        if view.password_edit.text() != view.confirm_password_edit.text():
            self.message_box.critical(view, view.wizard().app_name,
                                      PASSWORD_MISMATCH)
            return False
        return True

    def set_next_validation(self, view, backend):
        """Set the validation as the next page."""
        logger.debug('SetUpAccountController.set_next_validation')
        # validate the current info of the form, try to perform the action
        # to register the user, and then move foward
        if self.validate_form(view):
            backend.register_user(view.wizard().app_name, view.email,
                                  view.password, view.captcha_id,
                                  view.captcha_solution)
            view.next = view.wizard().email_verification_page_id
            view.wizard().next()

    def update_password_strength(self, password, view):
        """Callback used to update the password strenght UI code."""
        logger.debug('SetUpAccountController.update_password_strength')
        # get the strengh and then according to it color the frames
        strengh = get_password_strength(password)
        logger.info('Password strengh is %s', strengh)
        view.set_strenght_level(strengh, password)

    def is_correct_email_confirmation(self, email_address, view):
        """Return that the email is the same."""
        logger.debug('SetUpAccountController.is_correct_email_confirmation')
        return view.email_edit.text() == email_address

    def is_correct_password_confirmation(self, password, view):
        """Return that the passwords are correct."""
        logger.debug('SetUpAccountController.is_correct_password_confirmation')
        return view.password_edit.text() == password

    #pylint: disable=C0103
    @inlineCallbacks
    def setupUi(self, view):
        """Setup the view."""
        # request the backend to be used with the ui
        backend = yield self.get_backend()
        self._connect_ui_elements(view, backend)
        self._refresh_captcha(view, backend)
        self._set_titles(view)
        self._set_translated_strings(view)
        self._set_line_edits_validations(view)
    #pylint: enable=C0103


class TosController(object):
    """Controller used for the tos page."""

    def __init__(self, title='', subtitle='', tos_url=''):
        """Create a new instance."""
        self._title = title
        self._subtitle = subtitle   
        self._tos_url = tos_url

    #pylint: disable=C0103
    def setupUi(self, view):
        """Set up the ui."""
        view.setTitle(self._title)
        view.setSubTitle(self._subtitle)
        # load the tos page
        view.webkit.load(QUrl(self._tos_url))
    #pylint: enable=C0103


class EmailVerificationController(object):
    """Controller used for the verification page."""

    def _set_translated_strings(self, view):
        """Set the trnaslated strings."""
        logger.debug('EmailVerificationController._set_translated_strings')
        view.verification_code_edit.setPlaceholderText(VERIFY_EMAIL_TITLE)

    def _connect_ui_elements(self, view):
        """Set the connection of signals."""
        logger.debug('EmailVerificationController._connect_ui_elements')
        view.next_button.clicked.connect(lambda: self.next_page(view))

    def _set_titles(self, view):
        """Set the different titles."""
        logger.debug('EmailVerificationController._set_titles')
        view.setTitle(VERIFY_EMAIL_TITLE)
        view.setSubTitle(VERIFY_EMAIL_CONTENT)

    #pylint: disable=C0103
    def setupUi(self, view):
        """Setup the view."""
        self._set_titles(view)
        self._set_translated_strings(view)
        self._connect_ui_elements(view)
    #pylint: enable=C0103

    def next_page(self, view):
        """Call the next action."""
        logger.debug('EmailVerificationController.next_page')
        view.wizard().next()


class ErrorController(object):
    """Controller used for the error page."""

    def __init__(self):
        """Create a new instance."""

    #pylint: disable=C0103
    def setupUi(self, view):
        """Setup the view."""
        view.next = -1
        view.error_message_label.setText(ERROR)
    #pylint: enable=C0103


class ForgottenPasswordController(BackendController):
    """Controller used to deal with the forgotten pwd page."""

    def __init__(self):
        """Create a new instance."""
        super(ForgottenPasswordController, self).__init__()

    def _register_fields(self, view):
        """Register the fields of the wizard page."""
        view.registerField('email_address', view.email_address_line_edit)

    def _set_translated_strings(self, view):
        """Set the translated strings in the view."""
        view.forgotted_password_intro_label.setText(
                                    REQUEST_PASSWORD_TOKEN_LABEL % {'app_name':
                                    view.wizard().app_name})
        view.email_address_label.setText(EMAIL_LABEL)
        view.send_button.setText(RESET_PASSWORD)
        view.try_again_button.setText(TRY_AGAIN_BUTTON)

    def _set_enhanced_line_edir(self, view):
        """Set the extra logic to the line edits."""
        view.set_line_edit_validation_rule(view.email_address_line_edit,
                                           is_correct_email)

    def _connect_ui(self, view, backend):
        """Connect the diff signals from the Ui."""
        view.send_button.clicked.connect(
                            lambda: backend.request_password_reset_token(
                                                    view.wizard().app_name,
                                                    view.email_address))
        view.try_again_button.clicked.connect(lambda: self.on_try_again(view))
        # set the backend callbacks to be used
        backend.on_password_reset_token_sent_cb = lambda app, result:\
                                    self.on_password_reset_token_sent(view)
        backend.on_password_reset_error_cb = lambda app_name, error:\
                                    self.on_password_reset_error(app_name,
                                                                 error,
                                                                 view)

    def on_try_again(self, view):
        """Set back the widget to the initial state."""
        view.error_label.setVisible(False)
        view.try_again_widget.setVisible(False)
        view.email_widget.setVisible(True)

    def on_password_reset_token_sent(self, view):
        """Action taken when we managed to get the password reset done."""
        # ignore the result and move to the reset page
        view.next = view.wizard().reset_password_page_id
        view.wizard().next()

    def on_password_reset_error(self, app_name, error, view):
        """Action taken when there was an error requesting the reset."""
        if error['errtype'] == 'ResetPasswordTokenError':
            # the account provided is wrong, lets tell the user to try
            # again
            view.error_label.setText(REQUEST_PASSWORD_TOKEN_WRONG_EMAIL)
            view.error_label.setVisible(True)
        else:
            # ouch, I dont know what went wrong, lets tell the user to try later
            view.email_widget.setVisible(False)
            view.forgotted_password_intro_label.setVisible(False)
            view.try_again_wisget.setVisible(True)
            # set the error message
            view.error_label.setText(REQUEST_PASSWORD_TOKEN_TECH_ERROR)
            
    #pylint: disable=C0103
    @inlineCallbacks
    def setupUi(self, view):
        """Setup the view."""
        backend = yield self.get_backend()
        # hide the error label
        view.error_label.setVisible(False)
        view.try_again_widget.setVisible(False)
        self._set_translated_strings(view)
        self._connect_ui(view, backend)
        self._set_enhanced_line_edir(view)
        self._register_fields(view)
    #pylint: enable=C0103


class ResetPasswordController(BackendController):
    """Controller used to deal with reseintg the password."""

    def __init__(self):
        """Create a new instance."""
        super(ResetPasswordController, self).__init__()

    def _set_translated_strings(self, view):
        """Translate the diff strings used in the app."""
        view.reset_code_line_edit.setPlaceholderText(RESET_CODE_ENTRY)
        view.password_line_edit.setPlaceholderText(PASSWORD1_ENTRY)
        view.confirm_password_line_edit.setPlaceholderText(PASSWORD2_ENTRY)
        view.reset_password_button.setText(RESET_PASSWORD)
        view.setSubTitle(PASSWORD_HELP)

    def _connect_ui(self, view, backend):
        """Connect the different ui signals."""
        view.reset_password_button.clicked.connect(
                                lambda: self.set_new_password(view, backend))
        backend.on_password_changed_cb = lambda app, result:\
                                            self.on_password_changed(app,
                                                                     result,
                                                                     view)
        backend.on_password_change_error_cb = lambda app, error:\
                                        self.on_password_change_error(app,
                                                                      error,
                                                                      view)

    def _add_line_edits_validations(self, view):
        """Add the validations to be use by the line edits."""
        view.set_line_edit_validation_rule(view.password_line_edit,
                                           is_min_required_password)
        view.set_line_edit_validation_rule(view.confirm_password_line_edit,
                      lambda x: self.is_correct_password_confirmation(x, view))
        # same as the above case, lets connect a signal to a signal
        view.password_line_edit.textChanged.connect(
                              view.confirm_password_line_edit.textChanged.emit)

    def on_password_changed(self, app_name, result, view):
        """Let user know that the password was changed."""
    
    def on_password_change_error(self, app_name, error, view):
        """Let the user know that there was an error."""

    def set_new_password(self, view, backend):
        """Request a new password to be set."""
        app_name = view.wizard().app_name
        email = str(view.wizard().field('email_address').toString())
        code = view.reset_code
        logger.info('Settig new password for %s and email %s with code %s',
                    app_name, email, code)
        backend.set_new_password(app_name, email, code, view.password)

    def is_correct_password_confirmation(self, password, view):
        """Return if the password is correct."""
        return view.password_line_edit.text() == password
        
    #pylint: disable=C0103
    @inlineCallbacks
    def setupUi(self, view):
        """Setup the view."""
        backend = yield self.get_backend()
        self._set_translated_strings(view)
        self._connect_ui(view, backend)
        self._add_line_edits_validations(view)
    #pylint: enable=C0103


class SuccessController(object):
    """Controller used for the success page."""

    def __init__(self):
        """Create a new instance."""

    #pylint: disable=C0103
    def setupUi(self, view):
        """Setup the view."""
        view.next = -1
        view.success_message_label.setText(SUCCESS)
    #pylint: enable=C0103


class UbuntuSSOWizardController(object):
    """Controller used for the overall wizard."""

    def __init__(self, login_success_callback=NO_OP,
                 registration_success_callback=NO_OP,
                 user_cancellation_callback=NO_OP):
        """Create a new instance."""
        self.login_success_callback = login_success_callback
        self.registration_success_callback = registration_success_callback
        self.user_cancellation_callback = user_cancellation_callback

    def on_user_cancelation(self, view):
        """Process the cancel action."""
        logger.debug('UbuntuSSOWizardController.on_user_cancelation')
        self.user_cancellation_callback(view.app_name)
        view.close()

    @inlineCallbacks
    def on_login_success(self, app_name, email, view):
        """Process the success of a login."""
        logger.debug('UbuntuSSOWizardController.on_login_success')
        result = yield self.login_success_callback(app_name, email)
        logger.debug('Result from callback is %s', result)
        if result == 0:
            logger.info('Success in calling the given success_callback')
            self.show_success_message(view)
        else:
            logger.info('Error in calling the given success_callback')
            self.show_error_message(view)

    @inlineCallbacks
    def on_registration_success(self, app_name, email, view):
        """Process the success of a registration."""
        logger.debug('UbuntuSSOWizardController.on_registration_success')
        result = yield self.registration_success_callback(app_name, email)
        # TODO: what to do?
        logger.debug('Result from callback is %s', result)
        if result == 0:
            logger.info('Success in calling the given registration_callback')
            self.show_success_message(view)
        else:
            logger.info('Success in calling the given registration_callback')
            self.show_error_message(view)

    def show_success_message(self, view):
        """Show the success message in the view."""
        logger.info('Showing success message.')
        # get the id of the success page, set it as the next id of the
        # current page and let the wizard move to the next step
        view.currentPage().next = view.success_page_id
        view.next()
        # show the finish button but with a close message
        buttons_layout = []
        buttons_layout.append(QWizard.Stretch)
        buttons_layout.append(QWizard.FinishButton)
        view.setButtonLayout(buttons_layout)

    def show_error_message(self, view):
        """Show the error page in the view."""
        logger.info('Showing error message.')
        # similar to the success page but using the error id
        view.currentPage().next = view.error_page_id
        view.next()
        # show the finish button but with a close message
        buttons_layout = []
        buttons_layout.append(QWizard.Stretch)
        buttons_layout.append(QWizard.FinishButton)
        view.setButtonLayout(buttons_layout)

    #pylint: disable=C0103
    def setupUi(self, view):
        """Setup the view."""
        view.setWizardStyle(QWizard.ModernStyle)
        view.button(QWizard.CancelButton).clicked.connect(
                                        lambda: self.on_user_cancelation(view))
        view.loginSuccess.connect(
             lambda app, email: self.on_login_success(app, email, view))
        view.registrationSuccess.connect(
             lambda app, email: self.on_registration_success(app, email, view))
    #pylint: enable=C0103
